import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'package:flutter_mylibrary/widget/widget_template.dart';
import '../flutter_mylibrary.dart';
import 'base_view_state.dart';

mixin BaseRefreshListController<T> on BaseViewState{


  ///自定义第一页页数
  int firstPage() => startPage;

  ///自定义一页数量
  int pageSize() => pageLimit;

  /// 页面数据
  List<T> list = [];

  /// 当前页码
  int _currentPageNum;

  int get currentPageNum => _currentPageNum;


  int _totalNum;

  int get totalNum => _totalNum??0;

  set totalNum(int value) {
    _totalNum = value;
  }


  bool get hasMore => getRequestInfo().useLoadMore?totalNum > list.length? true : false : false;


  onCompleted(List<T> data) {}


  ///---------------------------------------------------数据获取逻辑区-------------------------------------------------------

  void _getData() async{

    HttpData httpData = await request((httpData) {
      if(currentPageNum == firstPage()){
        list.clear();
      }
      httpData.data = List<T>.from(httpData.data.map((e) => requestInfo?.handleRes(e)).toList());
      list.addAll(httpData.data);
    }, isSetError: currentPageNum == firstPage());

    if(isEmpty(httpData.httpError)){
      setIdle();
    }

    refreshController.finishRefresh(success: isEmpty(httpData.httpError));
    refreshController.finishLoad(success: isEmpty(httpData.httpError), noMore: !hasMore);
  }




  void requestWhenError(){
    if(viewState == ViewState.error){
      setBusy();
      _getData();
    }
  }




  ///------------------------------------------------------刷新，上拉加载----------------------------------------------------

  void refreshData() {
    _currentPageNum = firstPage();
    _getData();
  }


  /// 上拉加载更多
  void loadMore() {
    _currentPageNum = ++_currentPageNum;
    _getData();
  }




  ///------------------------------------------------------下拉刷新控件----------------------------------------------------

  ///获取一个公共的刷新控件(EasyRefresh)
  Widget getEasyRefreshWidget({ViewStateModelWidgetListBuilder<List<T>> widgetListBuilder, ViewStateModelWidgetBuilder<List<T>> builder, Widget emptyWidget, Widget busyWidget, bool showErrorWidget = true, bool showEmptyWidget = true, bool showBusyWidget = true}){
    Widget emptyErrorWidget = buildRefreshNoDataWidget(viewState: this, busyWidget: busyWidget, emptyDataWidget: emptyWidget, showErrorWidget: showErrorWidget, showEmptyWidget: showEmptyWidget, showBusyWidget: showBusyWidget);
    return EasyRefresh.custom(
      enableControlFinishRefresh: true,
      enableControlFinishLoad: true,
      taskIndependence: true,
      emptyWidget: emptyErrorWidget,
      slivers: isNotEmpty(widgetListBuilder)? widgetListBuilder(list) : [
        isEmpty(emptyErrorWidget)? isNotEmpty(builder)? builder(list) : SliverToBoxAdapter(child: Container(),) :SliverToBoxAdapter(child: Container(),),
      ],
      onRefresh: isBusy? null : () async{
        refreshData();
      },
      onLoad: (isBusy || !hasMore)? null : () async{
        loadMore();
      },
      controller: refreshController,
    );
  }
}