import 'package:flutter_mylibrary/base/request_info.dart';
import 'package:flutter_mylibrary/config/const.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/easy_dio.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/http/http_data.dart';
import 'package:flutter_mylibrary/utils/utils.dart';

abstract class BaseViewState{

  /// 防止页面销毁后,异步任务才完成,导致报错
  bool disposed = false;

  /// 当前的页面状态,默认为busy,可在viewModel的构造方法中指定;
  ViewState viewState = ViewState.busy;

  EasyRefreshController _refreshController = EasyRefreshController();

  set refreshController(EasyRefreshController easyRefreshController){
    this._refreshController = easyRefreshController;
  }

  ///获取请求的数据，一般用于简单的页面请求
  RequestInfo getRequestInfo();

  RequestInfo beforeRequest(RequestInfo requestInfo){
    return requestInfo;
  }

  RequestInfo requestInfo;


  EasyRefreshController get refreshController => _refreshController;

  bool isInit = false;


  ///重新请求当报错时
  void requestWhenError();

  ///---------------------------------------------页面状态------------------------------------------------------

  HttpError httpError;

  bool get isBusy => viewState == ViewState.busy;

  bool get isIdle => viewState == ViewState.idle;

  bool get isError => viewState == ViewState.error;


  ///---------------------------------------------设置页面状态------------------------------------------------------


  void setIdle();

  void setBusy();

  void setError(HttpError httpError);


  HttpError getEmptyDataError(data){
    HttpError httpError = HttpError(HttpErrorType.EmptyDataError, NO_DATA_MSG, logMsg: '空数据: $data');
    return httpError;
  }


  HttpError getParseDataError(e){
    HttpError httpError = HttpError(HttpErrorType.DataError, PARSE_ERROR_MSG, logMsg: '数据解析错误: $e', cause: e is Exception? e : null);
    return httpError;
  }

  void onError(HttpError httpError) {}


  ///---------------------------------------------通用请求方法------------------------------------------------------

  ///通常有下拉刷新请求，第一页之后不会再设置页面错误状态
  Future<HttpData> request(void handleHttpData(HttpData data), {bool isSetError = true}) async{

    requestInfo = beforeRequest(requestInfo??getRequestInfo());

    HttpData _httpData = await doRequest(requestInfo);

    var data = _httpData.data;

    if(isEmpty(_httpData.httpError)){
      if(isNotEmpty(data)){
        try{
          handleHttpData(_httpData);
        }catch(e){
          _httpData.httpError =  getParseDataError(e);
        }
      }else{
        _httpData.httpError = getEmptyDataError(data);
      }

      return _httpData;
    }else{
      if(isSetError){
        setError(_httpData.httpError);
      }
      return _httpData;
    }
  }


  Future<HttpData> doRequest(RequestInfo requestInfo) async{

    return EasyDio().request(
      requestInfo?.method,
      requestInfo?.url,
      data: requestInfo?.data,
      publicMethod: requestInfo?.publicMethod,
    );
  }


}



/// 页面状态类型
enum ViewState {
  idle,
  busy,
  error
}